#!/usr/bin/env node

const fs = require('fs-extra')
const path = require('path')
const cac = require('cac')
const log = require('npmlog')
const { DepCost, DepCostEvents } = require('../lib')
const requirePkg = require('../lib/require-pkg')

const cli = cac()

cli
  .command('[...pkgs]', 'Retrieve the time and space cost of a dependency.')
  .option('--cwd', 'Current working directory.')
  .option('-t, --track', 'Whether to keep temp directory.')
  .option(
    '-r, --latest-versions <latestVersions>',
    'Specify the count of latest versions.'
  )
  .option('-r, --record', 'Whether to update DEPCOST.md.')
  .option('-s, --versions <versions>', 'Select specific versions.')
  .option('-l, --log-level <logLevel>', 'log level of npmlog under the hood.')
  .option('-n, --npm-client <npmClient>', 'set npm client, defaults to npm.')
  .option('-d, --debug', 'Shortcut to set log level to "debug".')
  .action((pkgs, opts) => {
    opts.cwd = opts.cwd || process.cwd()

    if (opts.debug) {
      opts.logLevel = 'debug'
    }

    log.heading = 'depcost'
    log.level = opts.logLevel || 'warn'
    log.info('pkgs', pkgs)
    log.info('opts', opts)
    const program = new DepCost({
      ...opts,
      pkgs,
    })

    let isFirst = false

    const results = []

    program.on(DepCostEvents.message, result => {
      results.push(result)
      if (!isFirst) {
        isFirst = true
        console.log(`name\t\t\tinstall size\t\treuqire time`)
      }
      console.log(`${result.pkg}\t\t\t${result.size}\t\t${result.requireTime}`)
    })

    program.runAndEmit().then(() => {
      console.log(results)

      if (opts.record) {
        let content = ''

        if (pkgs.length === 0) {
          const packageJson = requirePkg(opts.cwd)
          content += `## ${packageJson.version}\n`
        }

        content += `
| name | install size | reuqire time |
| ---  | --- | --- |
${results.map(result =>
          `| ${result.pkg} | ${result.size} | ${result.requireTime} |`
        ).join('\n')}
        `
        const RECORD_FILE = 'DEPCOST.md'
        const recordFile = path.join(opts.cwd, RECORD_FILE)
        if (fs.existsSync(recordFile)) {
          content = `\n\n` + content
        }

        fs.writeFileSync(recordFile, content, { flag: 'a' })
      }

    }).catch(error => {
      process.exitCode = 1
      console.log(require('chalk').red(error.stack))
    })
  })

cli.help()
cli.version(require('../package.json').version)
cli.parse()
